#!/bin/sh

sh ~/sbcl/run-sbcl.sh <<EOF
(require 'asdf)
(sb-thread:join-thread
 (sb-thread:make-thread
  (lambda () (asdf:oos 'asdf:load-op 'delsegno :verbose nil))))

(setf (sb-ext:bytes-consed-between-gcs) (/ 12582912 2))

(let ((ref (with-open-file (in "results/reference-with-big-heap.sexp" :direction :input)
             (read in)))
      (pref nil))
  (dolist (benchmark (delsegno:list-benchmarks))
    (with-open-file (out (format nil "results-growing-heap/~A"
                                 (string-downcase benchmark))
                         :direction :output :if-exists :supersede)
      (with-open-file (out-wb (format nil "results-growing-heap/~A.wb"
                                      (string-downcase benchmark))
                              :direction :output :if-exists :supersede)
        (with-open-file (out-s1 (format nil "results-growing-heap/~A.s1"
                                        (string-downcase benchmark))
                                :direction :output :if-exists :supersede)
          (dotimes (i 2)
            (format t "~&;;; Running ~A with ~A threads...~%" benchmark (1+ i))
            (sb-ext:gc :full t)
            (setf (sb-alien:extern-alien "n_gc_threads" sb-alien:int) (1+ i))
            (let* ((results (delsegno:run-benchmark
                             benchmark :events '(:real-time :pgc-work-balance)))
                   (real-time (find :real-time results :key #'first))
                   (work-balance (find :pgc-work-balance results :key #'first)))
              ;; save results for one-thread
              (when (zerop i)
                (push (list benchmark (third (getf (cdr real-time) :mean)))
                      pref))
              ;; speedup against sequential version
              (let* ((refval (second (find benchmark ref :key 'car
                                           :test 'string-equal)))
                     (mean (third (getf (cdr real-time) :mean)))
                     (nmean (/ refval mean))
                     (stddev (third (getf (cdr real-time) :stddev)))
                     (nstddev (/ (* stddev nmean) mean)))
                (format out "~A ~F ~F~%" (1+ i) nmean nstddev))
              ;; work balance
              (format out-wb "~A ~F ~F~%"
                      (1+ i)
                      (first (getf (cdr work-balance) :mean))
                      (first (getf (cdr work-balance) :stddev)))
              ;; speedup against pgc w/ 1 thread
              (let* ((refval (second (find benchmark pref
                                           :key 'car :test 'string-equal)))
                     (mean (third (getf (cdr real-time) :mean)))
                     (nmean (/ refval mean))
                     (stddev (third (getf (cdr real-time) :stddev)))
                     (nstddev (/ (* stddev nmean) mean)))
                (format out-s1 "~A ~F ~F~%" (1+ i) nmean nstddev)))))))))
EOF
