(defpackage #:delsegno
  (:use #:cl)
  (:export #:define-benchmark #:run-benchmark #:run-all-benchmarks
           #:list-benchmarks)
  (:import-from #:perfpiece #:sample))

(in-package #:delsegno)

(defvar *benchmarks* nil)

(defmacro define-benchmark (name arguments &body body)
  `(progn
     (pushnew ',name *benchmarks*)
     (defun ,name ,arguments ,@body)))

(defun run-benchmark (name &key (events '(:real-time)))
  (let ((fn (find name *benchmarks* :test #'string-equal)))
    (if fn
        (sample (lambda ()
                  (sb-thread:join-thread
                   (sb-thread:make-thread fn)))
                :events events
                :samples 6
                :discard-first t
                :report :table-and-results)
        (warn "No such benchmark: ~A" name))))

(defun list-benchmarks ()
  (copy-list *benchmarks*))

(defun run-all-benchmarks ()
  (dolist (benchmark (list-benchmarks))
    (format t "~&;; Running ~A ...~%" benchmark)
    (run-benchmark benchmark)))
